@@ -1,4 +1,4 @@
-/* $Id: trans1.c,v 1.29.2.1 2001/11/25 16:48:30 karim Exp $
+/* $Id: trans1.c,v 1.29.2.5 2002/11/15 13:04:51 bill Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -796,17 +796,17 @@ gsqrt(GEN x, long prec)
       return padic_sqrt(x);
 
     case t_SER:
-      e=valp(x);
+      e = valp(x);
       if (gcmp0(x)) return zeroser(varn(x), (e+1)>>1);
       if (e & 1) err(sqrter6);
       av = avma;
-      /* trick: ser_pui assumes valp(x) = 0 */
-      y = ser_pui(x,ghalf,prec);
+      y = dummycopy(x); setvalp(y, 0);
+      y = ser_pui(y, ghalf, prec);
       if (typ(y) == t_SER) /* generic case */
         y[1] = evalsigne(1) | evalvalp(e>>1) | evalvarn(varn(x));
       else /* e.g coeffs are POLMODs */
-        y = gerepileupto(av, gmul(y, gpowgs(polx[varn(x)], e>>1)));
-      return y;
+        y = gmul(y, gpowgs(polx[varn(x)], e>>1));
+      return gerepileupto(av, y);
   }
   return transc(gsqrt,x,prec);
 }
@@ -1056,6 +1056,17 @@ gsqrtn(GEN x, GEN n, GEN *zetan, long prec)
     if (tx==t_INT && is_pm1(x) && signe(x)>0)
       y=gun;    /*speed-up since there is no way to call rootsof1complex
 		directly from gp*/
+    else if (gcmp0(x))
+    {
+      if (signe(n) < 0) err(gdiver2);
+      if (isinexactreal(x))
+      {
+        y=cgetr(2); 
+        y[1]=evalexpo(itos(gfloor(gdivsg(gexpo(x), n))));
+      }
+      else
+        y = realzero(prec);
+    }
     else
     {
       av=avma;
@@ -1521,7 +1532,7 @@ mpsc1(GEN x, long *ptmod8)
     n=(long)((1+c)/2.0);
     setexpo(p1,expo(p1)-m);
   }
-  else { m=0; n=(long)((1+beta/alpha)/2.0); }
+  else { m=0; n=(long)((2+beta/alpha)/2.0); }
   l2=l+1+(m>>TWOPOTBITS_IN_LONG);
   p2=realun(l2); setlg(p2,4);
   p4=cgetr(l2); av = avma;
@@ -1556,7 +1567,7 @@ mpsc1(GEN x, long *ptmod8)
     if (l4<=l2) { setlg(p2,l4); setlg(p4,l4); }
     subsrz(1,p3,p2); avma=av;
   }
-  if (l4<=l2) { setlg(p2,l4); setlg(p4,l4); }
+  setlg(p2,l2); setlg(p4,l2);
   setexpo(p4,expo(p4)-1); setsigne(p4, -signe(p4));
   p2 = mulrr(p4,p2);
   for (i=1; i<=m; i++)
